#!/usr/bin/env python

# original createsitessavannah.py by Marco Dias
# expanded to general purpose savannah submission by Paul Rossman
# 2009-01-25, initial version

# CMS Web login needs XHTMLFormParser to work
from ClientForm import XHTMLCompatibleFormParser
# Make a new DefaultFactory
from mechanize._html import DefaultFactory, FormsFactory
from mechanize import Browser
# Use optparse rather than getopt
from optparse import OptionParser
import sys, re, pprint

# Defaults used when submitting requests
hnusername = None
hnpassfile = None
assigned = 'cmscompinfrasup-facilities'
site = 'Unknown'
category = 'Facilities Operations'
summary = 'No summary provided'
details = 'No details provided'
login_page = 'https://savannah.cern.ch/account/login.php?uri=%2F'
create_ticket = 'https://savannah.cern.ch/support/?func=additem&group=cmscompinfrasup'

# Process arguments
parser = OptionParser()
#parser.set_defaults(verbose=True)
#parser.add_option('--quiet',
#                  action='store_false', dest='verbose', default=True,
#                  help='don\'t print status messages to stdout')
parser.add_option('--dump',
                  action='store_true', dest='dump', default=False,
                  help='dump contents of form controls on "Sumbit Item" page to stdout then exits, useful for debugging')
parser.add_option('-u', '--user', '--username', dest='hnusername', default=hnusername,
                  help='hypernews user account '
                  '[required]', metavar='USERNAME')
parser.add_option('-p', '--pfile', '--passfile', dest='hnpassfile', default=hnpassfile,
                  help='path to file containing the hypernews user account password '
                  '[required]', metavar='PASSFILE')
parser.add_option('-a', '--assigned', dest='assigned', default=assigned,
                  help='user or squad this request will be assigned to '
                  '[default: %default]', metavar='ASSIGNED')
parser.add_option('-s', '--site', dest='site', default=site,
                  help='CMS name of the site '
                  '[default: %default]', metavar='SITE')
parser.add_option('-c', '--category', dest='category', default=category,
                  help='savannah category '
                  '[default: %default]', metavar='CATEGORY')
parser.add_option('-m', '--summary', dest='summary', default=summary,
                  help='one line description for this request '
                  '[default: %default]', metavar='SUMMARY')
parser.add_option('-d', '--details', dest='details', default=details,
                  help='full description for this request '
                  '[default: %default]', metavar='DETAILS')
(options, args) = parser.parse_args()

if options.hnusername is None:
    parser.print_usage()
    print 'hypernews user account is required, use -h for help'
    exit(1) # exit the program

if options.hnpassfile is None:
    parser.print_usage()
    print 'path to file containing the hypernews user account password, use -h for help'
    exit(1) # exit the program

# Read the password file
try:
    f = open(options.hnpassfile, 'r')
    hnpassword = f.readline()
    f.close()
except IOError as (errno, strerror):
    print 'I/O error({0}): {1}'.format(errno, strerror)
    print 'unable to open the file "'+options.hnpassfile+'", please check the path and try again\n'
    exit(1) # exit the program
except:
    print "unexpected error:", sys.exc_info()[0]
    raise
    exit(1) # exit the program

# Setup the forms and browser objects
new_form_fac = FormsFactory(form_parser_class = XHTMLCompatibleFormParser)
new_def_fac = DefaultFactory()
new_def_fac._forms_factory = new_form_fac
br = Browser(factory = new_def_fac)

#br.set_handle_referer(True)
br.set_handle_refresh(True)
#br.set_handle_equiv(True)

## HTTP Error 403: request disallowed by robots.txt
br.set_handle_robots(False)

# Navigate to login page
br.open(login_page)

# search form is form 0, login form is form 1
br.select_form(nr=1)

br['form_loginname'] = options.hnusername
br['form_pw'] = hnpassword.strip()

# Submit form
response1 = br.submit()

# Various response methods
# print response1.geturl()
# print response1.info()  # headers
# print response1.read()  # body

# response1.read() successful login:
#        <li class="menutitle">
#           Logged in as rossman
#        </li><!-- end menutitle -->

# response1.read() unsuccessful login:
#        <li class="menuitem"> <span class="error">Not Logged In</span></li>

# Check to see if login was successful, if not exit
if not re.search('Logged in as ' + options.hnusername, response1.read()):
    print 'login unsuccessful, please check your username and password'
    exit(1) # exit the program

# Navigate to ticket creation
br.open(create_ticket)

# Select the correct form
br.select_form(name='trackers_form')

# If the dump option was enabled
if options.dump is True:
    pp = pprint.PrettyPrinter(indent=4)
    print '#' * 80
    print 'Category'
    print '-' * 80    
    pp.pprint(br.find_control('category_id').items)
    print '#' * 80
    print 'Assigned'
    print '-' * 80
    pp.pprint(br.find_control('assigned_to').items)
    print '#' * 80
    print 'Site'
    print '-' * 80
    pp.pprint(br.find_control('custom_sb1').items)
    print
    exit(0) # exit the program

# Some simple form control validation
if options.assigned in [item.attrs['label'] for item in br.find_control('assigned_to').items]:
    br.set_value_by_label([options.assigned], 'assigned_to')
else:
    # Use default
    br.set_value_by_label([assigned], 'assigned_to')

if options.site in [item.attrs['label'] for item in br.find_control('custom_sb1').items]:
    br.set_value_by_label([options.site], 'custom_sb1')
else:
    # Use default
    br.set_value_by_label([site], 'custom_sb1')

if options.category in [item.attrs['label'] for item in br.find_control('category_id').items]:
    br.set_value_by_label([options.category], 'category_id')
else:
    # Use default
    br.set_value_by_label([category], 'category_id')

# Fill in remaining form fields
br['summary'] = options.summary
br['details'] = options.details

# Submit form
response2 = br.submit()

# Check to see if login was successful, if not exit
if not re.search('Success:</span> New item posted', response2.read()):
    print 'form submit may have been unsuccessful, please verify request on Savannah'
    exit(1) # exit the program
