from sqlalchemy import *

from dataProviderBase import *
from T0Mon.utils import *
from T0Mon.consts import *
from T0Mon.sqlManager import *

from T0Mon.globals import logger
import T0Mon.config

### Module for classes providing data about T0 jobs ###

class RepackStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "Repack stats"):
        StatsDataProvider.__init__(self, title, TABLES.REPACK_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = getStatsColsRepack(self.tableName, self.runid)

    def _buildQuery(self, status):
        jobTable = Tbl(self.tableName)
        assocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        streamerTable = Tbl(TABLES.STREAMER)
        self.query = select(
            [func.count(distinct(jobTable.c.job_id))],
            and_(
                jobTable.c.job_status == status,
                streamerTable.c.run_id == self.runid
            ),
            from_obj = [
                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                    outerjoin(streamerTable, assocTable.c.streamer_id == streamerTable.c.streamer_id)
            ]
        )

class JobsStatsDataProvider(StatsDataProvider):
	def __init__(self, inputTier, title = "Stats", runid = None, mergeJobs = False ):
		self.title = title
		self.qtime = 0
		self.data = []
		self.data.append([])
		self.bindParams = None
		self.inputTier = inputTier
		self.runid = runid
                if mergeJobs:
                    self.mergeSign = ""
                else:
                    self.mergeSign = "NOT"

	def _buildCols(self):
		self.cols = getStatsColsMergeRecoAlca(self.inputTier, self.mergeSign, self.runid)

	def _buildQuery(self, status):
                wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
                wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
                wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
                wmbsDatasetAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
                datasetTable = Tbl(TABLES.DATASET_PATH, True)
                dataTierTable = Tbl(TABLES.DATA_TIER, True)


                fromFragment = """ INNER JOIN """ + wmbsJobAssocTable + """ assoc ON (j.id = assoc.job)
                                   INNER JOIN """ + wmbsDatasetAssocTable  + """ a ON (a.file_id = assoc.fileid)
                                   INNER JOIN """ + datasetTable  + """ dp ON (a.dataset_path_id = dp.id) """
                whereFragment = """ WHERE dp.data_tier = ( SELECT id FROM """ + dataTierTable + """ WHERE name='""" + self.inputTier + """' )
                                          AND j.name """ + self.mergeSign + """ LIKE 'Merge%' """
		if self.runid != None:
                        fromFragment += """ INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (assoc.fileid = m.fileid) """
                        whereFragment += """ and m.run = """ + str(self.runid)
 
                self.query = """SELECT COUNT(*)
                                       FROM (SELECT UNIQUE j.id FROM """ + wmbsJobTable + """ j """
                self.query += fromFragment
		if status == JOB_USED:
                        wmbsGroupJobAcquiredTable = Tbl(TABLES.WMBS_GROUP_JOB_ACQUIRED, True)
			self.query +=   """ INNER JOIN """ + wmbsGroupJobAcquiredTable + """ ja ON (j.id = ja.job) """
                elif status == JOB_SUCCESS:
                        wmbsGroupJobCompleteTable = Tbl(TABLES.WMBS_GROUP_JOB_COMPLETE, True)
                        self.query +=   """ INNER JOIN """ + wmbsGroupJobCompleteTable + """ jc ON (j.id = jc.job) """
                elif status == JOB_FAILURE:
                        wmbsGroupJobFailedTable = Tbl(TABLES.WMBS_GROUP_JOB_FAILED, True)
                        self.query +=   """ INNER JOIN """ + wmbsGroupJobFailedTable + """ jf ON (j.id = jf.job) """
		else: 
			self.query = """"""
                self.query += whereFragment + """ ) """

	def load(self):
		self._buildCols()
		for i in range (JOB_USED, JOB_FAILURE + 1):
			self._buildQuery(i)
			sqlResult = executeQuery(self.query, self.bindParams)
			self.data[0].append(sqlResult.fetchone()[0])
			self.qtime += sqlResult.time
			sqlResult.close()


class MergeStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "Repack Merge stats"):
        StatsDataProvider.__init__(self, title, TABLES.MERGE_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = getStatsColsMergeRecoAlca(self.tableName, self.runid)

    def _buildQuery(self, status):
        jobTable = Tbl(TABLES.MERGE_JOBS)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        repackedTable = Tbl(TABLES.REPACKED)
        self.query = select(
            [func.count(distinct(jobTable.c.job_id))],
            and_(
                jobTable.c.job_status == status,
                repackedTable.c.run_id == self.runid
            ),
            from_obj = [
                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                    outerjoin(repackedTable, assocTable.c.repacked_id == repackedTable.c.repacked_id)
            ]
        )
        # globals.logger.log(self.query)

class RecoStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "Reco stats"):
        StatsDataProvider.__init__(self, title, TABLES.RECO_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = getStatsCols(self.tableName, self.runid)

    def _buildQuery(self, status):
        jobTable = Tbl(TABLES.RECO_JOBS)
        assocTable = Tbl(TABLES.RECO_JOB_REPACK_ASSOC)
        repackedTable = Tbl(TABLES.REPACKED)
        self.query = select(
            [func.count(distinct(jobTable.c.job_id))],
            and_(
                jobTable.c.job_status == status,
                repackedTable.c.run_id == self.runid
            ),
            from_obj = [
                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                    outerjoin(repackedTable, assocTable.c.repacked_id == repackedTable.c.repacked_id)
            ]
        )
        # globals.logger.log(self.query)

class RecoMergeStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "Reco Merge stats"):
        StatsDataProvider.__init__(self, title, TABLES.MERGE_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = getStatsCols(self.tableName, self.runid)

    def _buildQuery(self, status):
        jobTable = Tbl(self.tableName)
        assocTable = Tbl(TABLES.MERGE_JOB_RECO_ASSOC)
        recoTable = Tbl(TABLES.RECONSTRUCTED)
        self.query = select(
            [func.count(distinct(jobTable.c.job_id))],
            and_(
                jobTable.c.job_status == status,
                recoTable.c.run_id == self.runid
            ),
            from_obj = [
                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                    outerjoin(recoTable, assocTable.c.reconstructed_id == recoTable.c.reconstructed_id)
            ]
        )

class AlcaskimStatsDataProvider(StatsDataProvider):
    def __init__(self, runid, title = "AlcaSkim stats"):
        StatsDataProvider.__init__(self, title, TABLES.ALCA_JOBS)
        self.runid = runid
        StatsDataProvider.load(self)

    def _buildCols(self):
        self.cols = getStatsCols(self.tableName, self.runid)

    def _buildQuery(self, status):
        jobTable = Tbl(self.tableName)
        assocTable = Tbl(TABLES.ALCA_JOB_RECO_ASSOC)
        recoTable = Tbl(TABLES.RECONSTRUCTED)
        self.query = select(
            [func.count(distinct(jobTable.c.job_id))],
            and_(
                jobTable.c.job_status == status,
                recoTable.c.run_id == self.runid
            ),
            from_obj = [
                jobTable.outerjoin(assocTable, assocTable.c.job_id == jobTable.c.job_id).
                    outerjoin(recoTable, assocTable.c.reconstructed_id == recoTable.c.reconstructed_id)
            ]
        )


class BaseRepackMergeJobDP(PagedTableDataProvider):

    def __init__(self, orderby = 0, asc = True, page = 0, pageSize = config.PAGE_SIZE):
        PagedTableDataProvider.__init__(self, orderby, asc, page, pageSize = pageSize)
        self.title = "Merge jobs"
        self.cols = Cols([
            Col("Job ID", decorator=buildRepackMergeJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Definition time", decorator=formatTime),
            Col("Repacked"),
            Col("Dataset"),
            Col("Status")
        ])

    def _buildCountQuery(self):
        jobTable = Tbl(TABLES.MERGE_JOBS)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        repackTable = Tbl(TABLES.REPACKED)
        datasetTable = Tbl(TABLES.PRIMARY_DATASET)
        self.countQuery = select(
            [func.count(func.distinct(jobTable.c.job_id))],
            self.whereCond,
            from_obj = [
                jobTable.
                outerjoin(assocTable, jobTable.c.job_id == assocTable.c.job_id).
                outerjoin(repackTable, repackTable.c.repacked_id == assocTable.c.repacked_id).
                outerjoin(datasetTable, datasetTable.c.dataset_id == repackTable.c.dataset_id)
            ]
        )

    def _buildMainQuery(self):
        jobTable = Tbl(TABLES.MERGE_JOBS)
        statusTable = Tbl(TABLES.JOB_STATUS)
        assocTable = Tbl(TABLES.MERGE_JOB_REPACK_ASSOC)
        repackTable = Tbl(TABLES.REPACKED)
        datasetTable = Tbl(TABLES.PRIMARY_DATASET)

        c = (
            jobTable.c.job_id,
            repackTable.c.run_id,
            jobTable.c.definition_time,
            func.count(assocTable.c.repacked_id).label("num_repacked"),
            datasetTable.c.name,
            statusTable.c.status
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
            from_obj = [
                jobTable.
                outerjoin(statusTable, statusTable.c.id == jobTable.c.job_status).
                outerjoin(assocTable, jobTable.c.job_id == assocTable.c.job_id).
                outerjoin(repackTable, repackTable.c.repacked_id == assocTable.c.repacked_id).
                outerjoin(datasetTable, datasetTable.c.dataset_id == repackTable.c.dataset_id)
            ]
            ).group_by(
                jobTable.c.job_id,
                jobTable.c.definition_time,
                datasetTable.c.name,
                repackTable.c.run_id,
                statusTable.c.status,
            ).order_by(o)


class BaseRecoJobDP(PagedTableDataProvider):

    def __init__(self, orderby = 0, asc = True, page = 0, pageSize = config.PAGE_SIZE):
        PagedTableDataProvider.__init__(self, orderby, asc, page, pageSize = pageSize)
        self.title = "Reco jobs"
        self.cols = Cols([
            Col("Job ID", decorator=buildRecoJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Definition time", decorator=formatTime),
            Col("Merged"),
            Col("Dataset"),
#            Col("Status"),     # TODO : insert again this column when I'd know the way the check the status
        ])

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j """ + self.fromCond + """ 
                                WHERE j.name LIKE 'PromptReco%' """ + self.whereCond + """ """

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        datasetRunStreamAssocTable = Tbl(TABLES.DATASET_RUN_STREAM_ASSOC, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.completion_time, count(a.fileid) as num_merged, pd.name
                                FROM """ + wmbsJobTable + """ j 
                                        INNER JOIN """ + wmbsJobAssocTable + """ a ON (a.job = j.id)
                                        INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (a.fileid = m.fileid)
                                        INNER JOIN """ + datasetRunStreamAssocTable + """ a2 ON (m.run = a2.run_id)
                                        INNER JOIN """ + primaryDatasetTable + """ pd ON (a2.primary_dataset_id = pd.id)
                                         """ + self.fromCond + """ 
                                WHERE j.name LIKE 'PromptReco%' """ + self.whereCond + """
                                GROUP BY j.id, j.completion_time, pd.name, m.run
                                ORDER BY """ + str(int(self.orderby)+1) + """ """ + order

        # TODO: check if this time is the correct one to show in the table ?????  --> COMPLETION_TIME


class BaseAlcaskimJobDP(PagedTableDataProvider):

    def __init__(self, orderby = 0, asc = True, page = 0, pageSize = config.PAGE_SIZE):
        PagedTableDataProvider.__init__(self, orderby, asc, page, pageSize = pageSize)
        self.title = "AlcaSkim jobs"
        self.cols = Cols([
            Col("Job ID", decorator=buildAlcaSkimJobLink),
            Col("Run ID", decorator=formRunLink),
#            Col("Definition time", decorator=formatTime),  
	    Col("Completion time", decorator=formatTime),    # TODO : remove this column
            Col("Merged"),
            Col("Dataset"),
#            Col("Status"),
        ])   # TODO : complete columns in table with the ones which were in ols version

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j """ + self.fromCond + """ 
                                WHERE j.name LIKE 'Alcaskim%' """ + self.whereCond + """ """

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        datasetRunStreamAssocTable = Tbl(TABLES.DATASET_RUN_STREAM_ASSOC, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.completion_time, count(a.fileid) as num_merged, pd.name
                                FROM """ + wmbsJobTable + """ j 
                                        INNER JOIN """ + wmbsJobAssocTable + """ a ON (a.job = j.id)
                                        INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (a.fileid = m.fileid)
                                        INNER JOIN """ + datasetRunStreamAssocTable + """ a2 ON (m.run = a2.run_id)
                                        INNER JOIN """ + primaryDatasetTable + """ pd ON (a2.primary_dataset_id = pd.id)
					 """ + self.fromCond + """ 
                                WHERE j.name LIKE 'Alcaskim%' """ + self.whereCond + """ 
                                GROUP BY j.id, j.completion_time, pd.name, m.run
                                ORDER BY """ + str(int(self.orderby)+1) + """ """ + order
	# TODO: check if this time is the correct one to show in the table ?????  --> COMPLETION_TIME


class BaseRepackJobDP(PagedTableDataProvider):
    def __init__(self, orderby = 0, asc = True, page = 0, pageSize = config.PAGE_SIZE):
        PagedTableDataProvider.__init__(self, orderby, asc, page, pageSize = pageSize)
        self.title = "Repacker jobs"
        self.cols = Cols([
            Col("Job ID", decorator=buildRepackJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Definition time", decorator=formatTime),
            Col("Streamers"),
            Col("Stream"),
            Col("Status"),
        ])

    def _buildCountQuery(self):
        jobTable = Tbl(TABLES.REPACK_JOBS)
        assocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        datasetTable = Tbl(TABLES.PRIMARY_DATASET)
        streamerTable = Tbl(TABLES.STREAMER)
        self.countQuery = select(
            [func.count(func.distinct(jobTable.c.job_id))],
            self.whereCond,
            from_obj = [
                jobTable.
                outerjoin(assocTable, jobTable.c.job_id == assocTable.c.job_id).
                outerjoin(streamerTable, streamerTable.c.streamer_id == assocTable.c.streamer_id)
#                outerjoin(datasetTable, datasetTable.c.dataset_id == assocTable.c.dataset_id)
            ]
        )

    def _buildMainQuery(self):
        jobTable = Tbl(TABLES.REPACK_JOBS)
        assocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        statusTable = Tbl(TABLES.JOB_STATUS)
        streamerTable = Tbl(TABLES.STREAMER)
	streamTable = Tbl(TABLES.STREAM)

        c = (
             jobTable.c.job_id,
             streamerTable.c.run_id,
             jobTable.c.definition_time,
             func.count(func.distinct(assocTable.c.streamer_id)).label("num_streamers"),
             streamTable.c.name,
             statusTable.c.status
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
            from_obj = [
                jobTable.
                outerjoin(statusTable, jobTable.c.job_status == statusTable.c.id).
                outerjoin(assocTable, jobTable.c.job_id == assocTable.c.job_id).
                outerjoin(streamerTable, streamerTable.c.streamer_id == assocTable.c.streamer_id).
                outerjoin(streamTable, streamerTable.c.stream_id == streamTable.c.id)
            ]
            ).group_by(
                jobTable.c.job_id,
                jobTable.c.definition_time,
                streamerTable.c.run_id,
                streamTable.c.name,
                statusTable.c.status,
            ).order_by(o)

class RepackJobByStreamerDP(BaseRepackJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRepackJobDP.__init__(self, *args, **kwds)
        assocTable = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        jobTable = Tbl(TABLES.REPACK_JOBS)
        self.whereCond = jobTable.c.job_id.in_(
            select([func.distinct(assocTable.c.job_id)], assocTable.c.streamer_id.in_(ids))
        )
        BaseRepackJobDP._load(self)

class RepackJobByIdsDP(BaseRepackJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRepackJobDP.__init__(self, *args, **kwds)
        jobTable = Tbl(TABLES.REPACK_JOBS)
        self.whereCond = jobTable.c.job_id.in_(ids)
        BaseRepackJobDP._load(self)

class RepackJobByRepackIdsDP(BaseRepackJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRepackJobDP.__init__(self, *args, **kwds)
        assocTable1 = Tbl(TABLES.JOB_DATASET_STREAMER_ASSOC)
        assocTable2 = Tbl(TABLES.REPACK_STREAMER_ASSOC)
        jobTable = Tbl(TABLES.REPACK_JOBS)
        self.whereCond = jobTable.c.job_id.in_(
            select([func.distinct(assocTable1.c.job_id)], 
			assocTable1.c.streamer_id.in_(select([assocTable2.c.streamer_id], assocTable2.c.repacked_id.in_(ids)))
            )
        )
        BaseRepackJobDP._load(self)


class RepackMergeJobByRepackIdsDP(BaseRepackMergeJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRepackMergeJobDP.__init__(self, *args, **kwds)
        self.cols = Cols([
            Col("Job ID", decorator=buildRepackMergeJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Definition time", decorator=formatTime),
            Col("Repacked"),
            Col("Dataset")
#            Col("Status")   ## How to get the status of the job?
        ])
	self.title = "Repack Merge jobs"
	self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
		BaseRepackMergeJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)

	self.countQuery = """ SELECT COUNT(DISTINCT j.id)
				FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                WHERE ja.fileid IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
	wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
	wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
	wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
	datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
	primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
	wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

	if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
				FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
				 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
				 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
				 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
				 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
				WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ WHERE fileid IN (""" + self.ids_str + """))
				GROUP BY j.id, m.run, j.last_update, pd.name
				ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class RecoJobByMergeIdsDP(BaseRecoJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
	self.cols[3] = Col("Files")
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
		BaseRecoJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                WHERE ja.fileid IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ WHERE fileid IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order

class RecoMergeJobByRecoIdsDP(BaseRecoJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
	self.title = "Reco Merge jobs"
	self.cols["Job ID"].decorator = buildRecoMergeJobLink
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
        	BaseRecoJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                WHERE ja.fileid IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ WHERE fileid IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class RecoMergeJobByIdsDP(BaseRecoJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
        self.title = "Reco Merge jobs"
        self.cols["Job ID"].decorator = buildRecoMergeJobLink
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseRecoJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)

        self.countQuery = """ SELECT COUNT(DISTINCT id)
                                FROM """ + wmbsJobTable + """ 
                                WHERE id IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE j.id IN (""" + self.ids_str + """)
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class RecoMergeJobByRecoMergeIdsDP(BaseRecoJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
        self.title = "Reco Merge jobs"
        self.cols["Job ID"].decorator = buildRecoMergeJobLink
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseRecoJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j 
                                WHERE j.id IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid) WHERE child IN (""" + self.ids_str + """))"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
	wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid) WHERE child IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class RecoJobByRecoIdsDP(BaseRecoJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
        self.cols[3] = Col("Files")
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseRecoJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j
                                WHERE j.id IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid)
 WHERE child IN (""" + self.ids_str + """))"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid) WHERE child IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class RepackRunningJobDP(BaseRepackJobDP):
    def __init__(self, *args, **kwds):
        kwds['pageSize'] = 25
        BaseRepackJobDP.__init__(self, *args, **kwds)
        self.cols[2] = Col("Running time", decorator=deltaTime)
        self.cols["Status"].visible = False
        jobTable = Tbl(TABLES.REPACK_JOBS)
        self.whereCond = or_(
            jobTable.c.job_status == JOB_USED,
            jobTable.c.job_status == JOB_NEW,
        )
        self.title = "Repack running jobs"
        BaseRepackJobDP._load(self)

class RepackMergeRunningJobDP(BaseRepackMergeJobDP):
    def __init__(self, *args, **kwds):
        kwds['pageSize'] = 25
        BaseRepackMergeJobDP.__init__(self, *args, **kwds)
	self.cols = Cols([
            Col("Job ID", decorator=buildRepackMergeJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Runing time", decorator=deltaTime),
            Col("Repacked"),
            Col("Dataset")
        ])
        self.cols[2] = Col("Running time", decorator=deltaTime)
        self.title = "Merge running jobs"
        BaseRepackMergeJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsGroupJobAcquiredTable = Tbl(TABLES.WMBS_GROUP_JOB_ACQUIRED, True)

	self.countQuery = """ SELECT COUNT(DISTINCT j.id)
				FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsGroupJobAcquiredTable + """ ac ON (j.id = ac.job)
				WHERE j.name LIKE 'Merge%' """

    def _buildMainQuery(self):
	wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
	wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
	wmbsGroupJobAcquiredTable = Tbl(TABLES.WMBS_GROUP_JOB_ACQUIRED, True)
	wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)
	primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
	datasetRunStreamAssocTable = Tbl(TABLES.DATASET_RUN_STREAM_ASSOC, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

	self.mainQuery = """ SELECT j.id, m.run, j.completion_time, count(a.fileid) as num_repacked, pd.name
				FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsGroupJobAcquiredTable + """ ac ON (j.id = ac.job)
					INNER JOIN """ + wmbsJobAssocTable + """ a ON (a.job = j.id)
					INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (a.fileid = m.fileid)
					INNER JOIN """ + datasetRunStreamAssocTable + """ a2 ON (m.run = a2.run_id)
					INNER JOIN """ + primaryDatasetTable + """ pd ON (a2.primary_dataset_id = pd.id)
				WHERE j.name LIKE 'Merge%'
				GROUP BY j.id, j.completion_time, pd.name, m.run
				ORDER BY """ + str(int(self.orderby)+1) + """ """ + order  

	# TODO: check if this time is the correct one to show in the table ?????  --> COMPLETION_TIME

class RecoRunningJobDP(BaseRecoJobDP):
    def __init__(self, *args, **kwds):
        kwds['pageSize'] = 25
        BaseRecoJobDP.__init__(self, *args, **kwds)
        self.title = "Reco running jobs"

        wmbsGroupJobAcquiredTable = Tbl(TABLES.WMBS_GROUP_JOB_ACQUIRED, True)

        self.fromCond = """ INNER JOIN """ + wmbsGroupJobAcquiredTable + """ ac ON (j.id = ac.job) """

        self.whereCond = """ """

        BaseRecoJobDP._load(self)

class AlcaskimRunningJobDP(BaseAlcaskimJobDP):
    def __init__(self, *args, **kwds):
        kwds['pageSize'] = 25
        BaseAlcaskimJobDP.__init__(self, *args, **kwds)
        self.title = "AlcaSkim running jobs"

        wmbsGroupJobAcquiredTable = Tbl(TABLES.WMBS_GROUP_JOB_ACQUIRED, True)

	self.fromCond = """ INNER JOIN """ + wmbsGroupJobAcquiredTable + """ ac ON (j.id = ac.job) """
	
	self.whereCond = """ """

        BaseAlcaskimJobDP._load(self)

class AlcaSkimJobByRecoMergeIdsDP(BaseAlcaskimJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseAlcaskimJobDP.__init__(self, *args, **kwds)
	self.cols[3] = Col("Files")
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
	        BaseAlcaskimJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                WHERE ja.fileid IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ WHERE fileid IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class AlcaSkimJobByAlcaSkimIdsDP(BaseAlcaskimJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseAlcaskimJobDP.__init__(self, *args, **kwds)
        self.cols[3] = Col("Files")
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseAlcaskimJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j
                                WHERE j.id IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid)
 WHERE child IN (""" + self.ids_str + """))"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid) WHERE child IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order




class AlcaSkimMergeJobByIdsDP(BaseAlcaskimJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseAlcaskimJobDP.__init__(self, *args, **kwds)
        self.title = "AlcaSkim Merge jobs"
        self.cols["Job ID"].decorator = buildAlcaSkimMergeJobLink
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseAlcaskimJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)

        self.countQuery = """ SELECT COUNT(DISTINCT id)
                                FROM """ + wmbsJobTable + """
                                WHERE id IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE j.id IN (""" + self.ids_str + """)
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class AlcaSkimMergeJobByAlcaSkimIdsDP(BaseAlcaskimJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseAlcaskimJobDP.__init__(self, *args, **kwds)
	self.title = "AlcaSkim Merge jobs"
	self.cols["Job ID"].decorator = buildAlcaSkimMergeJobLink
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseAlcaskimJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                WHERE ja.fileid IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ WHERE fileid IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order



class AlcaSkimMergeJobByAlcaSkimMergeIdsDP(BaseAlcaskimJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseAlcaskimJobDP.__init__(self, *args, **kwds)
        self.title = "AlcaSkim Merge jobs"
        self.cols["Job ID"].decorator = buildAlcaSkimMergeJobLink
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseAlcaskimJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j 
                                WHERE j.id IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid) WHERE child IN (""" + self.ids_str + """))"""


    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
	wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid) WHERE child IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class AlcaSkimJobByIdsDP(BaseAlcaskimJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseAlcaskimJobDP.__init__(self, *args, **kwds)
        self.cols[3] = Col("Files")
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseAlcaskimJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)

        self.countQuery = """ SELECT COUNT(DISTINCT id)
                                FROM """ + wmbsJobTable + """
                                WHERE id IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE j.id IN (""" + self.ids_str + """)
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order



class RepackJobByStatusDP(BaseRepackJobDP):

    def __init__(self, status, *args, **kwds):
        BaseRepackJobDP.__init__(self, *args, **kwds)
        self.cols["Status"].visible = False
        jobTable = Tbl(TABLES.REPACK_JOBS)
        self.whereCond = jobTable.c.job_status == status
        s = JOB_STATUS_REPACKED[status]
        self.title = "Repacker " + s + " jobs"
        BaseRepackJobDP._load(self)

class RepackMergeJobByStatusDP(BaseRepackMergeJobDP):

    def __init__(self, status, *args, **kwds):
        BaseRepackMergeJobDP.__init__(self, *args, **kwds)
        self.cols["Status"].visible = False
        jobTable = Tbl(TABLES.MERGE_JOBS)
        self.whereCond = jobTable.c.job_status == status
        s = JOB_STATUS[status]
        self.title = "Merge " + s + " jobs"
        BaseRepackMergeJobDP._load(self)

class RecoJobByStatusDP(BaseRecoJobDP):
    def __init__(self, status, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
        self.cols["Status"].visible = False
        jobTable = Tbl(TABLES.RECO_JOBS)
        self.whereCond = jobTable.c.job_status == status
        s = JOB_STATUS[status]
        self.title = "Reco " + s + " jobs"
        BaseRecoJobDP._load(self)

class RecoJobByIdsDP(BaseRecoJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
        self.cols[3] = Col("Files")
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseRecoJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)

        self.countQuery = """ SELECT COUNT(DISTINCT id)
                                FROM """ + wmbsJobTable + """ 
                                WHERE id IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE j.id IN (""" + self.ids_str + """)
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order


class RepackJobByStatusAndRunIdDP(BaseRepackJobDP):
    def __init__(self, status, runid, *args, **kwds):
        BaseRepackJobDP.__init__(self, *args, **kwds)
        self.cols["Status"].visible = False
        streamerTable = Tbl(TABLES.STREAMER)
        jobTable = Tbl(TABLES.REPACK_JOBS)
        s = JOB_STATUS_REPACK[status]
        self.title = "Repacker " + s + " jobs for run " + str(runid)
        BaseRepackJobDP._load(self)

class RepackMergeJobByRepackMergeIdsDP(BaseRepackMergeJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRepackMergeJobDP.__init__(self, *args, **kwds)
        self.cols = Cols([
            Col("Job ID", decorator=buildRepackMergeJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Definition time", decorator=formatTime),
            Col("Repacked"),
            Col("Dataset")
#            Col("Status")   ## How to get the status of the job?
        ])
        self.title = "Repack Merge jobs"
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseRepackMergeJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)

        self.countQuery = """ SELECT COUNT(DISTINCT j.id)
                                FROM """ + wmbsJobTable + """ j 
                                WHERE j.id IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid)
 WHERE child IN (""" + self.ids_str + """))"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
	wmbsFileParentTable = Tbl(TABLES.WMBS_FILE_PARENT, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE ja.job IN (SELECT job FROM """ + wmbsJobAssocTable + """ INNER JOIN """ + wmbsFileParentTable + """ ON (parent = fileid) WHERE child IN (""" + self.ids_str + """))
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order



class RepackMergeJobByIdsDP(BaseRepackMergeJobDP):
    def __init__(self, ids, *args, **kwds):
        BaseRepackMergeJobDP.__init__(self, *args, **kwds)
        self.cols = Cols([
            Col("Job ID", decorator=buildRepackMergeJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Definition time", decorator=formatTime),
            Col("Repacked"),
            Col("Dataset")
#            Col("Status")   ## How to get the status of the job?
        ])
        self.title = "Repack Merge jobs"
        self.ids = ids
        self.ids_str = ""
        numIds = len(self.ids)
        if numIds != 0:
                if numIds > 1:
                        for i in range(numIds-1):
                                self.ids_str += str(self.ids[i]) + ", "
                self.ids_str += str(self.ids[-1])
                BaseRepackMergeJobDP._load(self)

    def _buildCountQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)

        self.countQuery = """ SELECT COUNT(DISTINCT id)
                                FROM """ + wmbsJobTable + """ 
                                WHERE id IN (""" + self.ids_str + """)"""

    def _buildMainQuery(self):
        wmbsJobTable = Tbl(TABLES.WMBS_JOB, True)
        wmbsJobAssocTable = Tbl(TABLES.WMBS_JOB_ASSOC, True)
        wmbsFileDetailsTable = Tbl(TABLES.WMBS_FILE_DETAILS, True)
        wmbsFileDatasetPathAssocTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC, True)
        datasetPathTable = Tbl(TABLES.DATASET_PATH, True)
        primaryDatasetTable = Tbl(TABLES.PRIMARY_DATASET, True)
        wmbsFileRunlumiMapTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP, True)

        if self.asc:
                order = "ASC"
        else:
                order = "DESC"

        self.mainQuery = """ SELECT j.id, m.run, j.last_update, COUNT(DISTINCT ja.fileid), pd.name
                                FROM """ + wmbsJobTable + """ j INNER JOIN """ + wmbsJobAssocTable + """ ja ON (j.id = ja.job)
                                 INNER JOIN """ + wmbsFileDetailsTable + """ f ON (ja.fileid = f.id)
                                 INNER JOIN """ + wmbsFileDatasetPathAssocTable + """ a ON (a.file_id = f.id)
                                 INNER JOIN """ + datasetPathTable + """ dp ON (dp.id = a.dataset_path_id)
                                 INNER JOIN """ + primaryDatasetTable + """ pd ON (pd.id = dp.primary_dataset)
                                 INNER JOIN """ + wmbsFileRunlumiMapTable + """ m ON (f.id = m.fileid)
                                WHERE j.id IN (""" + self.ids_str + """)
                                GROUP BY j.id, m.run, j.last_update, pd.name
                                ORDER BY  """  + str(int(self.orderby)+1) + """ """ + order

class RepackMergeJobByStatusAndRunIdDP(BaseRepackMergeJobDP):

    def __init__(self, status, runid, *args, **kwds):
        BaseRepackMergeJobDP.__init__(self, *args, **kwds)
        self.cols["Status"].visible = False
        repackedTable = Tbl(TABLES.REPACKED)
        jobTable = Tbl(TABLES.MERGE_JOBS)
        self.whereCond = and_(
            jobTable.c.job_status == status,
            repackedTable.c.run_id == runid,
        )
        s = JOB_STATUS_REPACK[status]
        self.title = "Merger " + s + " jobs for run " + str(runid)
        BaseRepackMergeJobDP._load(self)

class RecoJobByStatusAndRunIdDP(BaseRecoJobDP):

    def __init__(self, status, runid, *args, **kwds):
        BaseRecoJobDP.__init__(self, *args, **kwds)
        self.cols["Status"].visible = False
        repackedTable = Tbl(TABLES.REPACKED)
        jobTable = Tbl(TABLES.RECO_JOBS)
        self.whereCond = and_(
            jobTable.c.job_status == status,
            repackedTable.c.run_id == runid,
        )
        s = JOB_STATUS[status]
        self.title = "Reco " + s + " jobs for run " + str(runid)
        BaseRecoJobDP._load(self)

class BaseWMBSJobDP(PagedTableDataProvider):

    def __init__(self, status, orderby = 0, asc = True, page = 0, pageSize = config.PAGE_SIZE):
        PagedTableDataProvider.__init__(self, orderby, asc, page, pageSize = pageSize)
        self.title = "WMBS jobs"
        self.status = status
        self.cols = Cols([
            Col("Job ID", decorator=buildRepackJobLink),
            Col("Run ID", decorator=formRunLink),
            Col("Definition time", decorator=formatTime),
            Col("# of files"), 
            Col("Dataset"),
            Col("Status")
        ])

    def _buildCountQuery(self):
        jobTable = Tbl(TABLES.WMBS_JOB)
        statusTable = Tbl( "WMBS_GROUP_JOB_" + self.status )
        assocTable = Tbl(TABLES.WMBS_JOB_ASSOC)
        fileTable = Tbl(TABLES.WMBS_FILE_DETAILS)
        fileRunLumiTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP)
        fileDatasetPathTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC)
        datasetPathTable = Tbl(TABLES.DATASET_PATH)
        datasetTable = Tbl(TABLES.PRIMARY_DATASET)
        dataTierTable = Tbl(TABLES.DATA_TIER)
        self.countQuery = select(
            [func.count(func.distinct(jobTable.c.id))],
            self.whereCond,
            from_obj = [
                jobTable.
                join(statusTable, jobTable.c.id == statusTable.c.job).
                outerjoin(assocTable, jobTable.c.id == assocTable.c.job).
                outerjoin(fileTable, fileTable.c.id == assocTable.c.fileid).
                outerjoin(fileRunLumiTable, fileTable.c.id == fileRunLumiTable.c.fileid).
                outerjoin(fileDatasetPathTable, fileTable.c.id == fileDatasetPathTable.c.file_id).
                outerjoin(datasetPathTable, fileDatasetPathTable.c.dataset_path_id == datasetPathTable.c.id).
                outerjoin(datasetTable, datasetTable.c.id == datasetPathTable.c.primary_dataset).
                outerjoin(dataTierTable, dataTierTable.c.id == datasetPathTable.c.data_tier)
            ]
        )

    def _buildMainQuery(self):
        jobTable = Tbl(TABLES.WMBS_JOB)
        statusTable = Tbl( "WMBS_GROUP_JOB_" + self.status )
        assocTable = Tbl(TABLES.WMBS_JOB_ASSOC)
        fileTable = Tbl(TABLES.WMBS_FILE_DETAILS)
        fileRunLumiTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP)
        fileDatasetPathTable = Tbl(TABLES.WMBS_FILE_DATASET_PATH_ASSOC)
        datasetPathTable = Tbl(TABLES.DATASET_PATH)
        datasetTable = Tbl(TABLES.PRIMARY_DATASET)
        dataTierTable = Tbl(TABLES.DATA_TIER)
        c = (
             jobTable.c.id,
             fileRunLumiTable.c.run,
             jobTable.c.last_update,
             func.count(func.distinct(assocTable.c.fileid)).label("Input Files"),
             datasetTable.c.name
        )

        if self.asc: o = c[self.orderby].asc()
        else: o = c[self.orderby].desc()

        self.mainQuery = select(
            c,
            self.whereCond,
            from_obj = [
                jobTable.
                join(statusTable, jobTable.c.id == statusTable.c.job).
                outerjoin(assocTable, jobTable.c.id == assocTable.c.job).
                outerjoin(fileTable, fileTable.c.id == assocTable.c.fileid).
                outerjoin(fileRunLumiTable, fileTable.c.id == fileRunLumiTable.c.fileid).
                outerjoin(fileDatasetPathTable, fileTable.c.id == fileDatasetPathTable.c.file_id).
                outerjoin(datasetPathTable, fileDatasetPathTable.c.dataset_path_id == datasetPathTable.c.id).
                outerjoin(datasetTable, datasetTable.c.id == datasetPathTable.c.primary_dataset).
                outerjoin(dataTierTable, dataTierTable.c.id == datasetPathTable.c.data_tier)
            ]
            ).group_by(
                jobTable.c.id,
                jobTable.c.last_update,
                fileRunLumiTable.c.run,
                datasetTable.c.name
            ).order_by(o)

class WMBSJobByStatusDP(BaseWMBSJobDP):

    def __init__(self, inputTier, mergeFlag, status, *args, **kwds):
        BaseWMBSJobDP.__init__(self, status, *args, **kwds)
        self.cols["Status"].visible = False
        jobTable = Tbl(TABLES.WMBS_JOB)
        dataTierTable = Tbl(TABLES.DATA_TIER)
        if mergeFlag == "NOT":
            self.whereCond = and_(
                dataTierTable.c.name == inputTier,
                not_( jobTable.c.name.like("Merge%"))
                )
        else:
            self.whereCond = and_(
                dataTierTable.c.name == inputTier,
                jobTable.c.name.like("Merge%")
                )
        s = JOB_STATUS[status]
        t = JOB_TYPE[inputTier][mergeFlag]
        self.title = t + " " + s + " jobs"
        BaseWMBSJobDP._load(self)

class WMBSJobByStatusAndRunIdDP(BaseWMBSJobDP):

    def __init__(self, inputTier, mergeFlag, status, runid, *args, **kwds):
        BaseWMBSJobDP.__init__(self, status, *args, **kwds)
        self.cols["Run ID"].visible = False
        self.cols["Status"].visible = False
        jobTable = Tbl(TABLES.WMBS_JOB)
        fileRunLumiTable = Tbl(TABLES.WMBS_FILE_RUNLUMI_MAP)
        dataTierTable = Tbl(TABLES.DATA_TIER)
        if mergeFlag == "NOT":
            self.whereCond = and_(
                dataTierTable.c.name == inputTier,
                fileRunLumiTable.c.run == runid,
                not_( jobTable.c.name.like("Merge%"))
                )
        else:
            self.whereCond = and_(
                dataTierTable.c.name == inputTier,
                fileRunLumiTable.c.run == runid,
                jobTable.c.name.like("Merge%")
                )
        t = JOB_TYPE[inputTier][mergeFlag]
        s = JOB_STATUS[status]
        self.title = t+ " " + s + " jobs for run " + str(runid)
        BaseWMBSJobDP._load(self)

